==> org-lsst-ccs-subsystem-fcs-main-4.0.2-sources.jar: org/lsst/ccs/subsystems/fcs/FilterManager.java <==

package org.lsst.ccs.subsystems.fcs;

import java.util.ArrayList;

import java.util.List;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;
import org.lsst.ccs.command.annotations.Command;
import org.lsst.ccs.framework.Module;
import static org.lsst.ccs.subsystems.fcs.FCSCst.FCSLOG;
import org.lsst.ccs.subsystems.fcs.FcsEnumerations.FilterLocation;

/**
 * This class's goal is to manage the filters.
 * @author virieux
 */
public class FilterManager extends Module {
    
    
    private final List<Filter> filters = new ArrayList<>(); 

    
    /**
     * A map to store the filters that FCS manages. 
     * The key of this map is the filter name.
     * 
     */
    protected TreeMap<String, Filter> filtersMapByName;
    
    /**
     * A map to store the filters that FCS manages. 
     * The key of this map is the filter ID.
     * 
     */
    private TreeMap<Integer,Filter> filtersMapByID;

    /**
     * A getter for the Filters Map sorted by filter name.
     * @return 
     */
    public SortedMap<String, Filter> getFiltersMapByName() {
        return (TreeMap<String, Filter>) filtersMapByName.clone();
    }

    /**
     * 
     * @return 
     */
    public SortedMap<Integer, Filter> getFiltersMapByID() {
        return (TreeMap<Integer, Filter>)filtersMapByID.clone();
    }
    
    
    
    
    @Override
    public void initModule() {
        /** By construction, FilterManager's sons are the filters.**/
        Map<String, Filter> map = getSubsystem().getComponentLookup().getChildren(getName(), Filter.class);
        filters.addAll(map.values());
        
        /** 
        * Creation of the HashMap filtersMap.
        * To move this initialization to the constructor is a bad idea. 
        * It has been tested and it doesn't work because in the construction phase
        * the environment of the objects Filter is not set, so the method getName() 
        * returns null.
        */
        filtersMapByName = new TreeMap<>();
        filtersMapByID = new TreeMap<>();
        for (Filter f: filters) {

            FCSLOG.info("Add filter to filterManager:" + f.toString());
            filtersMapByName.put(f.getName(), f);
            filtersMapByID.put(f.getFilterID(), f);
        }
    }
    
    /**
     * Return a filter which name is given as argument.
     * @param filterName
     * @return 
     */
    public Filter getFilterByName(String filterName) {
        return filtersMapByName.get(filterName);
    }
    
    /**
     * Return Return filter which id is given as argument.
     * @param id
     * @return 
     */
    public Filter getFilterByID(int id) {
        return filtersMapByID.get(id);
    }
    
    /**
     * Return Return filter name which correspond to ID.
     * @param id
     * @return 
     */
    @Command(type = Command.CommandType.QUERY, level = Command.NORMAL,
        description = "Return filter name which correspond to ID.")
    public String getFilterNameByID(int id) {
        if (filtersMapByID.containsKey(id)) {
            return filtersMapByID.get(id).getName();
        } else {
            return null;
        }
    }
    
    
    /**
     * Returns a list of filter names. This list is sorted by filter ID.
     * Useful for initialization of the GUI.
     * @return 
     */
    public List<String> getFilterNamesSortedByFilterID() {
        ArrayList<String> namesList = new ArrayList<>();
        TreeMap<Integer,String> filterNamesByID = new TreeMap<>();
        for (Map.Entry<String,Filter> entry: filtersMapByName.entrySet()) {
            Filter filter = entry.getValue();
            filterNamesByID.put(filter.getFilterID(), filter.getName());
        }
        namesList.addAll(filterNamesByID.values());
        return namesList;
    }
    
    /**
     * This method controls if the filter name given at the console by an
     * operator is correct.
     *
     * @param filterName
     * @throws IllegalArgumentException in this case :
     * <UL>
     * <LI>the name is not a String,</LI>
     * <LI>the name is unknown in the map filters,</LI>
     * <LI>the name is known in the map filters, but the filter location is
     * unknown,</LI>
     * <LI>the name is known in the map filters, but the filter is out of the
     * camera.</LI>
     * </UL>
     */
    protected void checkFilterName(String filterName) {
      
        if (this.getFilterByName(filterName) == null) {
            throw new IllegalArgumentException("Unknown filter name : " + filterName);
            
        } else if (getFilterByName(filterName).isUnknown()) {
            throw new IllegalArgumentException("Filter: " + filterName + " is at unknown location.");
            
        } else if (this.getFilterByName(filterName).isOut()) {
            throw new IllegalArgumentException("Filter: " + filterName + " is out of the camera.");
        }
    }
    
    /**
     * For the filter which id is given as argument, set a new location given as argument.
     * @param filterID
     * @param filterLocation 
     */
    public void setFilterLocation(int filterID, FilterLocation filterLocation) {
        Filter filter = filtersMapByID.get(filterID);
        if (filter == null) {
            throw new IllegalArgumentException("Unknown filter ID : " + filterID);
        } else {
            filter.setFilterLocation(filterLocation);
        }
    }
    
    /**
     * Returns name of filter on autochanger trucks if there is a filter on trucks,
     * returns null otherwise.
     * @return 
     */
    @Command(type = Command.CommandType.QUERY, level = Command.NORMAL,
        description = "Returns name of filter on autochanger trucks if there is a filter on trucks,null otherwise.")    
    public String getFilterOnAutochangerName() {
        String filterName = null;
        for (Map.Entry<String,Filter> entry: filtersMapByName.entrySet()) {
            Filter filter = entry.getValue();
            if ("AC".equals(filter.getSocketName())) {
                return entry.getKey();
            }
        }
        return filterName;
    }
    
    /**
     * Publish filters data to refresh GUI.
     * 
     */
     @Command(type = Command.CommandType.QUERY, level = Command.NORMAL,
        description = "Publish filters data to refresh GUI.")   
    public void publishData() {
        for (Map.Entry<String,Filter> entry: filtersMapByName.entrySet()) {
            Filter filter = entry.getValue();
            filter.publishData();
        }
    }
    
    

    @Command(type = Command.CommandType.QUERY, level = Command.NORMAL,
            description = "Return a printed list of filters.", alias = "printFilterList")
    @Override
    public String toString() {
        return filtersMapByName.toString();
    }

}

==> org-lsst-ccs-subsystem-fcs-main-5.0.3-sources.jar: org/lsst/ccs/subsystems/fcs/FilterManager.java <==

package org.lsst.ccs.subsystems.fcs;

import java.util.ArrayList;

import java.util.List;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;
import org.lsst.ccs.command.annotations.Command;
import org.lsst.ccs.commons.annotations.LookupField;
import org.lsst.ccs.commons.annotations.LookupField.Strategy;
import org.lsst.ccs.commons.annotations.LookupName;
import org.lsst.ccs.framework.HasLifecycle;
import static org.lsst.ccs.subsystems.fcs.FCSCst.FCSLOG;
import org.lsst.ccs.subsystems.fcs.FcsEnumerations.FilterLocation;

/**
 * This class's goal is to manage the filters.
 * @author virieux
 */
public class FilterManager implements HasLifecycle {
    
    @LookupName
    private String name;

    @LookupField(strategy=Strategy.CHILDREN)
    private final List<Filter> filters = new ArrayList<>(); 

    
    /**
     * A map to store the filters that FCS manages. 
     * The key of this map is the filter name.
     * 
     */
    @LookupField(strategy=Strategy.CHILDREN)
    protected final TreeMap<String, Filter> filtersMapByName = new TreeMap<>();
    
    /**
     * A map to store the filters that FCS manages. 
     * The key of this map is the filter ID.
     * 
     */
    private final TreeMap<Integer,Filter> filtersMapByID = new TreeMap<>();

    /**
     * A getter for the Filters Map sorted by filter name.
     * @return 
     */
    public SortedMap<String, Filter> getFiltersMapByName() {
        return (SortedMap<String, Filter>) filtersMapByName.clone();
    }

    /**
     * 
     * @return 
     */
    public SortedMap<Integer, Filter> getFiltersMapByID() {
        return (TreeMap<Integer, Filter>)filtersMapByID.clone();
    }
    
    @Override
    public void init() {
        /** 
        * Creation of the HashMap filtersMap.
        * To move this initialization to the constructor is a bad idea. 
        * It has been tested and it doesn't work because in the construction phase
        * the environment of the objects Filter is not set, so the method getName() 
        * returns null.
        */
        for (Filter f: filters) {
            FCSLOG.info("Add filter to filterManager:" + f.toString());
            filtersMapByID.put(f.getFilterID(), f);
        }
    }
    
    /**
     * Return a filter which name is given as argument.
     * @param filterName
     * @return 
     */
    public Filter getFilterByName(String filterName) {
        return filtersMapByName.get(filterName);
    }
    
    /**
     * Return Return filter which id is given as argument.
     * @param id
     * @return 
     */
    public Filter getFilterByID(int id) {
        return filtersMapByID.get(id);
    }
    
    /**
     * Return Return filter name which correspond to ID.
     * @param id
     * @return 
     */
    @Command(type = Command.CommandType.QUERY, level = Command.NORMAL,
        description = "Return filter name which correspond to ID.")
    public String getFilterNameByID(int id) {
        if (filtersMapByID.containsKey(id)) {
            return filtersMapByID.get(id).getName();
        } else {
            FCSLOG.warning(name + ": " + id + "  unknown filter ID. Please update filterManager configuration.");
            return "filterID " + id;
        }
    }
    
    
    /**
     * Returns a list of filter names. This list is sorted by filter ID.
     * Useful for initialization of the GUI.
     * @return 
     */
    public List<String> getFilterNamesSortedByFilterID() {
        ArrayList<String> namesList = new ArrayList<>();
        TreeMap<Integer,String> filterNamesByID = new TreeMap<>();
        for (Map.Entry<String,Filter> entry: filtersMapByName.entrySet()) {
            Filter filter = entry.getValue();
            filterNamesByID.put(filter.getFilterID(), filter.getName());
        }
        namesList.addAll(filterNamesByID.values());
        return namesList;
    }
    
    /**
     * This method controls if the filter name given at the console by an
     * operator is correct.
     *
     * @param filterName
     * @throws IllegalArgumentException in this case :
     * <UL>
     * <LI>the name is not a String,</LI>
     * <LI>the name is unknown in the map filters,</LI>
     * <LI>the name is known in the map filters, but the filter location is
     * unknown,</LI>
     * <LI>the name is known in the map filters, but the filter is out of the
     * camera.</LI>
     * </UL>
     */
    protected void checkFilterName(String filterName) {
      
        if (this.getFilterByName(filterName) == null) {
            throw new IllegalArgumentException("Unknown filter name : " + filterName);
            
        } else if (getFilterByName(filterName).isUnknown()) {
            throw new IllegalArgumentException("Filter: " + filterName + " is at unknown location.");
            
        } else if (this.getFilterByName(filterName).isOut()) {
            throw new IllegalArgumentException("Filter: " + filterName + " is out of the camera.");
        }
    }
    
    /**
     * For the filter which id is given as argument, set a new location given as argument.
     * @param filterID
     * @param filterLocation 
     */
    public void setFilterLocation(int filterID, FilterLocation filterLocation) {
        Filter filter = filtersMapByID.get(filterID);
        if (filter == null) {
            throw new IllegalArgumentException("Unknown filter ID : " + filterID);
        } else {
            filter.setFilterLocation(filterLocation);
        }
    }
    
    /**
     * Returns name of filter on autochanger trucks if there is a filter on trucks,
     * returns null otherwise.
     * @return 
     */
    @Command(type = Command.CommandType.QUERY, level = Command.NORMAL,
        description = "Returns name of filter on autochanger trucks if there is a filter on trucks,null otherwise.")    
    public String getFilterOnAutochangerName() {
        String filterName = null;
        for (Map.Entry<String,Filter> entry: filtersMapByName.entrySet()) {
            Filter filter = entry.getValue();
            if ("AC".equals(filter.getSocketName())) {
                return entry.getKey();
            }
        }
        return filterName;
    }
    
    /**
     * Publish filters data to refresh GUI.
     * 
     */
     @Command(type = Command.CommandType.QUERY, level = Command.NORMAL,
        description = "Publish filters data to refresh GUI.")   
    public void publishData() {
        for (Map.Entry<String,Filter> entry: filtersMapByName.entrySet()) {
            Filter filter = entry.getValue();
            filter.publishData();
        }
    }
    
    

    @Command(type = Command.CommandType.QUERY, level = Command.NORMAL,
            description = "Return a printed list of filters.", alias = "printFilterList")
    @Override
    public String toString() {
        return filtersMapByName.toString();
    }

}
